//
// Négyzetrácsos sakktábla
// Felhasználási feltételek: Nevezd meg! - Így add tovább! (Creative Commons)
// Uray M. János
// 2013-2014
//

#include <iostream>
#include <string>
#include <vector>
#include <cstring>
#include <list>
#include <cassert>
#include <utility>

using namespace std;

int main (int n_args, char const* args[]) {
	if (n_args < 4) {
		cerr << "Használat:\n";
		cerr << args[0] << " <szélesség (db)> <magasság (db)> <mezőméret (pt)> <paraméterek>\n";
		cerr << "További paraméterek:\n";
		cerr << "   -edge=<szegély (pt)>\n";
		cerr << "   -margin=<x-margó (pt)>,<y-margó (pt)>\n";
		cerr << "   -a1=(white|black)\n";
		cerr << "   -fields=<mezőfelsorolás (0|1-sorozat)>\n";
		cerr << "Például: 8 8 64 -edge=4\n";
		return 1;
	}
	//
	// paraméterek
	//
	vector<string> sargs(args, args + n_args);
	int const files = stoi(sargs[1]);
	int const ranks = stoi(sargs[2]);
	int const size = stoi(sargs[3]);
	int const width = files * size;
	int const height = ranks * size;
	int edge = 0, margin_x = 0, margin_y = 0;
	vector<bool> fields;
	bool back = false;
	for (unsigned i = 4; i < sargs.size(); i++) {
		if (strncmp(sargs[i].c_str(), "-edge=", 6) == 0) {
			edge = stoi(sargs[i].substr(6));
		} else if (strncmp(sargs[i].c_str(), "-margin=", 8) == 0) {
			size_t idx;
			margin_x = margin_y = stoi(sargs[i].substr(8), &idx);
			if (sargs[i].size() > 8 + idx)
				margin_y = stoi(sargs[i].substr(8 + idx + 1));
		} else if (strncmp(sargs[i].c_str(), "-a1=", 4) == 0) {
			back = sargs[i][4] == 'w';
		} else if (strncmp(sargs[i].c_str(), "-fields=", 8) == 0) {
			if (sargs[i].size() != 8 + files * ranks)
				return cerr << "-fields: Nem megfelelő számú mező.\n", 2;
			fields.resize(files * ranks);
			for (unsigned j = 0; j < fields.size(); j++)
				fields[j] = sargs[i][8 + j] != '0';
		} else {
			return cerr << "Hibás paraméter: " << sargs[i] << "\n", 2;
		}
	}
	margin_x = std::max(margin_x, edge);
	margin_y = std::max(margin_y, edge);
	//
	// keret előállítása (ha van -fields)
	//
	typedef pair<int, int> point;
	typedef pair<point, point> line;
	list<line> lines;
	if (edge && !fields.empty()) {
		for (int y = 0; y <= ranks; y++) {
			for (int x = 0; x < files; x++) {
				bool above = y > 0 && fields[(y-1)*files + x];
				bool below = y < ranks && fields[y*files + x];
				if (above != below)
					lines.push_back(line(point(x, y), point(x+1, y)));
			}
		}
		for (int y = 0; y < ranks; y++) {
			for (int x = 0; x <= files; x++) {
				bool left  = x > 0 && fields[y*files + (x-1)];
				bool right = x < files && fields[y*files + x];
				if (left != right)
					lines.push_back(line(point(x, y), point(x, y+1)));
			}
		}
	}
	//
	// kiírás
	//
	cout << "<?xml version='1.0' encoding='UTF-8' standalone='no'?>\n";
	cout << "<!--\n";
	cout << "\tA képet a rect.cpp generálta a következő paraméterekkel:\n";
	for (unsigned i = 1; i < sargs.size(); i++)
		cout << (i == 1 ? "\t" : " ") << sargs[i];
	cout << "\n";
	cout << "\tFelhasználási feltételek: Nevezd meg! - Így add tovább! (Creative Commons)\n";
	cout << "\tUray M. János\n";
	cout << "\t2013-2014\n";
	cout << "-->\n";
	cout << "<!DOCTYPE svg PUBLIC '-//W3C//DTD SVG 1.1//EN' 'http://www.w3.org/Graphics/SVG/1.1/DTD/svg11.dtd'>\n";
	cout << "<svg width='" << width + 2*margin_x << "' height='" << height + 2*margin_y << "' viewBox='" << -margin_x << " " << -margin_y << " " << width + 2*margin_x << " " << height + 2*margin_y << "' xmlns='http://www.w3.org/2000/svg' xmlns:xlink='http://www.w3.org/1999/xlink'>\n";
	if (edge) {
		cout << "\t\n";
		cout << "\t<!-- keret -->\n";
		if (fields.empty()) {
			cout << "\t<rect fill='none' stroke='#666' stroke-width='" << edge*2 << "' stroke-linejoin='miter' x='0' y='0' width='" << width << "' height='" << height << "'/>\n";
		} else {
			while (!lines.empty()) {
				cout << "\t<polygon fill='none' stroke='#666' stroke-width='" << edge*2 << "' stroke-linejoin='miter' points='";
				auto line_it = lines.begin();
				line line = *line_it;
				point begin = line.first;
				point p = begin;
				do {
					cout << p.first * size << "," << p.second * size << " ";
					lines.erase(line_it);
					p = p == line.first ? line.second : line.first;
					if (p != begin) {
						auto it = lines.begin();
						while (it != lines.end() && !(it->first == p || it->second == p))
							it++;
						assert(it != lines.end());
						line_it = it;
						line = *line_it;
					}
				} while (p != begin);
				cout << "'/>\n";
			}
		}
	}
	cout << "\t\n";
	cout << "\t<!-- mezők -->\n";
	for (int y = 0; y < ranks; y++) {
		for (int x = 0; x < files; x++) {
			if (!fields.empty() && !fields[y*files + x])
				continue;
			bool black = (x + (ranks - y - 1)) % 2 == back;
			cout << "\t<rect fill='" << (black ? "#AAA" : "#FFF") << "' x='" << x*size << "' y='" << y*size << "' width='" << size << "' height='" << size << "'/>\n";
		}
	}
	cout << "\t\n";
	cout << "</svg>\n";
}

